import json
import numpy
import numpy as np
from typing import Any, Dict, List, Tuple, Union
import pandas as pd

SUCCESS = "success"

# unbiased estimator from https://github.com/openai/human-eval
def estimate_pass_at_k(
    num_samples: Union[int, List[int], np.ndarray],
    num_correct: Union[List[int], np.ndarray],
    k: int,
) -> np.ndarray:
    """
    Estimates pass@k of each problem and returns them in an array.
    """

    def estimator(n: int, c: int, k: int) -> float:
        """
        Calculates 1 - comb(n - c, k) / comb(n, k).
        """
        if n - c < k:
            return 1.0
        return 1.0 - np.prod(1.0 - k / np.arange(n - c + 1, n + 1))

    if isinstance(num_samples, int):
        num_samples_it = itertools.repeat(num_samples, len(num_correct))
    else:
        assert len(num_samples) == len(num_correct)
        num_samples_it = iter(num_samples)

    return np.array(
        [estimator(int(n), int(c), k) for n, c in zip(num_samples_it, num_correct)]
    )

def count(path):
    with open(path, "r") as f:
        results = json.load(f)

    total = np.array([r["nfiles"] for r in results["eval"].values()])
    base_correct = []
    new_correct = []

    for res in results["eval"].values():
        bc = sum([r[0] == SUCCESS for r in res["base"]])
        base_correct.append(bc)
        if res["plus"]:
            new_correct.append(
                sum(
                    [
                        res["plus"][i][0] == res["base"][i][0] == SUCCESS
                        for i in range(len(res["plus"]))
                    ]
                )
            )
    base_correct = np.array(base_correct)

    pass_at_k = [
        {"k": k, "pass@k": estimate_pass_at_k(total, base_correct, k).mean()}
        for k in range(1, total.min() + 1)
        if total.min() >= k
    ]

    with open("result.jsonl", "w") as f:
        for r in pass_at_k:
            f.write(json.dumps(r) + '\n')
