import pandas as pd
from pathlib import Path
import logging
import json

from tqdm import tqdm
from transformers import AutoTokenizer
from dataclasses import dataclass

codellamas_path = Path("/lustre/models/CodeLlama_240112/")

models_path = {
    "7b": "codellama-CodeLlama-7b-hf",
    "13b": "codellama-CodeLlama-13b-hf",
    "34b": "CodeLlama-34b-hf",
}


CODE_EOS = 28956 # '```' in codellama's tokenizer
EOS = 2 # default eos
STOP_TOKENS = [CODE_EOS, EOS]

def mk_prompt(question: str,  answer: str) -> str:
    return question + answer

def count_token(encode, cache, r):
    program = r["completion"]
    question = cache[r["task_id"]]
    return len(encode(mk_prompt(question, program)))

def main(model_name, sample_path):
    model_path = codellamas_path / models_path[model_name]
    tokenizer = AutoTokenizer.from_pretrained(model_path)

    with open("../testsets/humaneval/humaneval_prompt.jsonl", "r") as f:
        lines = [json.loads(l) for l in f.readlines()]
    question_cache = {l["task_id"]: l["prompt"] for l in lines}
        

    programs = pd.read_json(sample_path, lines=True)
    count_token_p = lambda r: count_token(tokenizer.encode, question_cache, r)
    programs["token_num"] = programs.apply(count_token_p, axis=1)
    print(programs["token_num"].mean())


if __name__ == "__main__":
    import argparse
    argparser = argparse.ArgumentParser()
    argparser.add_argument("name", type=str)
    argparser.add_argument("path", type=str)
    args = argparser.parse_args()

    logging.basicConfig(format="%(asctime)s %(name)s:%(levelname)s:%(message)s",
                        datefmt="%d-%M-%Y %H:%M:%S",
                        encoding='utf-8',
                        level=logging.INFO)

    logging.info(f"START: Testing codellama-{args.name}")
    main(args.name, args.path)
