from dataclasses import dataclass, asdict
import json
from pathlib import Path
import shutil

PROJ = Path("/lustre/S/nanziyuan/projects/llmtest")
EXP = PROJ / "exps"
TEST = PROJ / "testsets"

def get_llm_path(llm_name):
    codellama_path = Path("/lustre/models/CodeLlama_240112/")
    codellama70b_path = Path("/lustre/S/nanziyuan/projects/llmtest/models")
    llm_dicts = {
        "7b_base": codellama_path / "codellama-CodeLlama-7b-hf",
        "7b_python": codellama_path / "codellama-CodeLlama-7b-Python-hf",
        "7b_instr": codellama_path / "CodeLlama-7b-Instruct-hf",

        "13b_base": codellama_path / "codellama-CodeLlama-13b-hf",
        "13b_python": codellama_path / "codellama-CodeLlama-13b-Python-hf",
        "13b_instr": codellama_path / "CodeLlama-13b-Instruct-hf",

        "34b_base": codellama_path / "CodeLlama-34b-hf",
        "34b_python": codellama_path / "codellama-CodeLlama-34b-Python-hf",
        "34b_instr": codellama_path / "CodeLlama-34b-Instruct-hf",

        "70b_base": codellama70b_path / "70b_base",
        "70b_python": codellama70b_path / "70b_python",
        "70b_instr": codellama70b_path / "70b_instruct",
    }
    return llm_dicts[llm_name].as_posix()

def get_testset_path(testset_name):
    testset_dicts = {
        "apps_subset": TEST / "apps_subset" / "apps_subset_prompt.jsonl",
        "humaneval":  TEST / "humaneval" / "humaneval_prompt.jsonl",
    }
    return testset_dicts[testset_name].as_posix()

@dataclass
class Config:
    testset_path: str
    llm_path: str
    completion_dir: str
    k: int
    num_gpus: int
    stop: list[str]
    testset_start: int = 0

def save_config(cfg: Config, path: Path):
    with open(path, "w") as f:
        json.dump(asdict(cfg), f)

def mk_slurm(llm_name, gpu_num, cfg_path, slurm_dir):
    with open("gpu.slurm.template", "r") as f:
        template = f.read()

    slurm_script = template.format(
        name=llm_name,
        gpu_num=gpu_num,
        cfg_path=cfg_path.absolute()
    )

    with open(slurm_dir / "gpu.slurm", "w") as f:
        f.write(slurm_script)

    shutil.copy("./run.sh", slurm_dir / "run.sh")


def mk_config(llm_name, testset_name, exp_id):
    path = EXP / f"{llm_name}-{testset_name}-{exp_id:0>3}"
    path.mkdir()
    completion_dir = path / "completion/"
    completion_dir.mkdir()
    slurm_dir = path / "slurm"
    slurm_dir.mkdir()

    num_gpus = 4
    cfg = Config(
        testset_path=get_testset_path(testset_name),
        llm_path=get_llm_path(llm_name),
        completion_dir=completion_dir.as_posix(),
        k=1000,
        num_gpus=num_gpus,
        stop=["\nclass", "\ndef", "\n#", "\n@", "\nprint", "\nif", "\n```"],
    )

    cfg_path = path / "config.json"
    save_config(cfg, cfg_path)

    mk_slurm(llm_name, num_gpus, cfg_path, slurm_dir)

if __name__ == "__main__":
    mk_config("7b_base", "humaneval", 1)
